package com.nisum.application.service;


import com.nisum.application.dto.ProfileDTO;
import com.nisum.application.exception.ProfileConflictException;
import com.nisum.application.exception.ProfileNotFoundException;
import com.nisum.application.model.Profile;
import com.nisum.application.repository.ProfileRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.web.server.ResponseStatusException;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeParseException;
import java.util.regex.Pattern;


@Service
@Slf4j
public class ProfileService {

    @Autowired
    private ProfileRepository profileRepository;

    public Mono<Profile> createProfile(Profile profile) {
        // Ensure either PAN or Aadhar is provided
        if (profile.getPermanentAccountNumber() == null && profile.getAdharNumber() == null) {
            return Mono.error(new ResponseStatusException(HttpStatus.BAD_REQUEST, "Either PAN or Aadhar must be provided."));
        }

        // Validate Aadhar number
        if (profile.getAdharNumber() != null && !profile.getAdharNumber().matches("^[1-9][0-9]{11}$")) {
            return Mono.error(new ResponseStatusException(HttpStatus.BAD_REQUEST, "Aadhar must be a 12 digit number starting with 1-9."));
        }

        // Validate PAN number
        if (profile.getPermanentAccountNumber() != null && !profile.getPermanentAccountNumber().matches("^[A-Z]{5}[0-9]{4}[A-Z]$")) {
            return Mono.error(new ResponseStatusException(HttpStatus.BAD_REQUEST, "Invalid PAN format."));
        }

        // Check for PAN and Aadhar conflicts individually
        Mono<Profile> panConflictCheck = profile.getPermanentAccountNumber() != null
                ? profileRepository.findByPermanentAccountNumber(profile.getPermanentAccountNumber())
                .flatMap(existingProfile -> Mono.error(new ProfileConflictException("Profile with the same PAN already exists.")))
                : Mono.empty();

        Mono<Profile> aadharConflictCheck = profile.getAdharNumber() != null
                ? profileRepository.findByAdharNumber(profile.getAdharNumber())
                .flatMap(existingProfile -> Mono.error(new ProfileConflictException("Profile with the same Aadhar already exists.")))
                : Mono.empty();

        // Combine the two checks using `Mono.firstWithSignal` to propagate the first error or complete both if no conflicts
        return Mono.firstWithSignal(panConflictCheck, aadharConflictCheck)
                .switchIfEmpty(profileRepository.save(profile))
                .doOnSuccess(savedProfile -> log.info("Profile created successfully: {}", savedProfile))
                .doOnError(error -> log.error("Error creating profile", error));
    }

    public Flux<Profile> searchProfiles(String firstName, String lastName, String pan, String adhar, String mobile, LocalDate dob) {
        // Check if at least one search criteria is provided
        if (firstName == null && lastName == null && pan == null && adhar == null && mobile == null && dob == null) {
            log.error("Search criteria are missing. At least one criteria must be provided.");
            return Flux.error(new IllegalArgumentException("At least one search criteria must be provided."));
        }

        // Build the query based on provided criteria
        Flux<Profile> result;

        if (firstName != null || lastName != null) {
            String firstNameRegex = firstName != null ? ".*" + Pattern.quote(firstName) + ".*" : ".*";
            String lastNameRegex = lastName != null ? ".*" + Pattern.quote(lastName) + ".*" : ".*";
            result = profileRepository.findByFirstNameRegexAndLastNameRegex(firstNameRegex, lastNameRegex);
        } else if (pan != null) {
            result = profileRepository.findAllByPermanentAccountNumber(pan);
        } else if (adhar != null) {
            result = profileRepository.findAllByAdharNumber(adhar);
        } else if (mobile != null) {
            result = profileRepository.findByMobileNumber(mobile);
        } else if (dob != null) {
            result = profileRepository.findByDateOfBirth(dob);
        } else {
            result = Flux.empty();
        }

        // Log the search criteria and results
        return result
                .doOnSubscribe(subscription -> log.info("Search initiated with criteria - FirstName: {}, LastName: {}, PAN: {}, Aadhar: {}, Mobile: {}, DOB: {}",
                        firstName, lastName, pan, adhar, mobile, dob))
                .doOnNext(profile -> log.debug("Found profile: {}", profile))
                .doOnError(error -> log.error("Error searching profiles", error))
                .doOnComplete(() -> log.info("Search completed with criteria - FirstName: {}, LastName: {}, PAN: {}, Aadhar: {}, Mobile: {}, DOB: {}",
                        firstName, lastName, pan, adhar, mobile, dob))
                .switchIfEmpty(Flux.error(new IllegalStateException("No profiles found matching the given criteria.")));
    }

    public Mono<Profile> updateProfile(ProfileDTO updateDTO) {
        log.info("Received request to update profile with ID: {}", updateDTO.getProfileId());

        return profileRepository.findById(updateDTO.getProfileId())
                .flatMap(existingProfile -> {
                    boolean updated = false;

                    if (updateDTO.getFirstName() != null) {
                        existingProfile.setFirstName(updateDTO.getFirstName());
                        updated = true;
                    }
                    if (updateDTO.getLastName() != null) {
                        existingProfile.setLastName(updateDTO.getLastName());
                        updated = true;
                    }
                    if (updateDTO.getDob() != null) {
                        try {
                            existingProfile.setDateOfBirth(LocalDate.parse(updateDTO.getDob(), DateTimeFormatter.ofPattern("MM-dd-yyyy")));
                            updated = true;
                        } catch (DateTimeParseException e) {
                            log.error("Invalid date format: {}", updateDTO.getDob());
                            return Mono.error(new IllegalArgumentException("Invalid date format. Expected MM-DD-YYYY"));
                        }
                    }
                    if (updateDTO.getPan() != null) {
                        existingProfile.setPermanentAccountNumber(updateDTO.getPan());
                        updated = true;
                    }
                    if (updateDTO.getAdhar() != null) {
                        existingProfile.setAdharNumber(updateDTO.getAdhar());
                        updated = true;
                    }
                    if (updateDTO.getMobile() != null) {
                        existingProfile.setMobileNumber(updateDTO.getMobile());
                        updated = true;
                    }

                    if (!updated) {
                        log.warn("No fields to update for profile with ID: {}", updateDTO.getProfileId());
                        return Mono.error(new IllegalArgumentException("At least one field must be provided to update the profile."));
                    }

                    return profileRepository.save(existingProfile)
                            .doOnSuccess(updatedProfile -> log.info("Profile updated successfully: {}", updatedProfile))
                            .doOnError(error -> log.error("Error updating profile", error));
                })
                .switchIfEmpty(Mono.error(new IllegalArgumentException("Profile not found with the given ID")));
    }

    public Mono<Void> deleteProfileById(String profileId) {
        return profileRepository.findById(profileId)
                .switchIfEmpty(Mono.error(new ProfileNotFoundException("Profile not found with the given ID: " + profileId)))
                .flatMap(existingProfile -> profileRepository.delete(existingProfile));
    }

}



